@@ -1,36 +1,30 @@
-use v5.10;
-
 package Mac::PropertyList;
 use strict;
 
 use warnings;
 no warnings;
 
-use vars qw($ERROR);
+use vars qw($ERROR $XML_head $XML_foot $VERSION @EXPORT_OK %EXPORT_TAGS);
 use Carp qw(croak carp);
 use Data::Dumper;
-use HTML::Entities;
-use XML::Entities qw(decode_entities);
+use XML::Entities;
 
-use Exporter qw(import);
+use parent qw(Exporter);
 
-our @EXPORT_OK = qw(
+@EXPORT_OK = qw(
 	parse_plist
 	parse_plist_fh
 	parse_plist_file
 	plist_as_string
 	create_from_hash
 	create_from_array
-	create_from_string
 	);
 
-our %EXPORT_TAGS = (
+%EXPORT_TAGS = (
 	'all' => \@EXPORT_OK,
 	);
 
-our $VERSION = '1.603_05';
-
-=encoding utf8
+$VERSION = '1.35';
 
 =head1 NAME
 
@@ -38,10 +32,9 @@ Mac::PropertyList - work with Mac plists at a low level
 
 =head1 SYNOPSIS
 
-	use Mac::PropertyList qw(:all);
+	use Mac::PropertyList;
 
 	my $data  = parse_plist( $text );
-	my $perl  = $data->as_perl;
 
 		# == OR ==
 	my $data  = parse_plist_file( $filename );
@@ -63,10 +56,9 @@ Mac::PropertyList - work with Mac plists at a low level
 =head1 DESCRIPTION
 
 This module is a low-level interface to the Mac OS X Property List
-(plist) format in either XML or binary. You probably shouldn't use
-this in applications–build interfaces on top of this so you don't have
-to put all the heinous multi-level object stuff where people have to
-look at it.
+(plist) format. You probably shouldn't use this in applications--build
+interfaces on top of this so you don't have to put all the heinous
+multi-level object stuff where people have to look at it.
 
 You can parse a plist file and get back a data structure. You can take
 that data structure and get back the plist as XML. If you want to
@@ -75,14 +67,6 @@ change the structure inbetween that's your business. :)
 You don't need to be on Mac OS X to use this. It simply parses and
 manipulates a text format that Mac OS X uses.
 
-If you need to work with the old ASCII or newer JSON formet, you can
-use the B<plutil> tool that comes with MacOS X:
-
-	% plutil -convert xml1 -o ExampleBinary.xml.plist ExampleBinary.plist
-
-Or, you can extend this module to handle those formats (and send a pull
-request).
-
 =head2 The Property List format
 
 The MacOS X Property List format is simple XML. You can read the DTD
@@ -90,7 +74,7 @@ to get the details.
 
 	http://www.apple.com/DTDs/PropertyList-1.0.dtd
 
-One big problem exists—its dict type uses a flat structure to list
+One big problem exists--its dict type uses a flat structure to list
 keys and values so that values are only associated with their keys by
 their position in the file rather than by the structure of the DTD.
 This problem is the major design hinderance in this module. A smart
@@ -116,16 +100,9 @@ There are several types of objects:
 	Mac::PropertyList::data
 	Mac::PropertyList::real
 	Mac::PropertyList::integer
-	Mac::PropertyList::uid
 	Mac::PropertyList::date
 	Mac::PropertyList::array
 	Mac::PropertyList::dict
-	Mac::PropertyList::true
-	Mac::PropertyList::false
-
-Note that the Xcode property list editor abstracts the C<true> and
-C<false> objects as just C<Boolean>. They are separate tags in the
-plist format though.
 
 =over 4
 
@@ -158,12 +135,23 @@ information or blessed objects.
 
 my $Debug = $ENV{PLIST_DEBUG} || 0;
 
+$XML_head =<<"XML";
+<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+XML
+
+$XML_foot =<<"XML";
+</plist>
+XML
+
 my %Readers = (
 	"dict"    => \&read_dict,
 	"string"  => \&read_string,
 	"date"    => \&read_date,
 	"real"    => \&read_real,
 	"integer" => \&read_integer,
+	"string"  => \&read_string,
 	"array"   => \&read_array,
 	"data"    => \&read_data,
 	"true"    => \&read_true,
@@ -181,8 +169,6 @@ will be imported unless you ask for it.
 
 	use Mac::PropertyList qw( :all );
 
-=head2 Things that parse
-
 =over 4
 
 =item parse_plist( TEXT )
@@ -199,7 +185,6 @@ sub parse_plist {
 
 	my $plist = do {
 		if( $text =~ /\A<\?xml/ ) { # XML plists
-			$text =~ s/<!--(?:[\d\D]*?)-->//g;
 			# we can handle either 0.9 or 1.0
 			$text =~ s|^<\?xml.*?>\s*<!DOC.*>\s*<plist.*?>\s*||;
 			$text =~ s|\s*</plist>\s*$||;
@@ -266,7 +251,7 @@ sub parse_plist_file {
 		return;
 		}
 
-	my $text = do { local $/; open my($fh), '<:raw', $file; <$fh> };
+	my $text = do { local $/; open my($fh), $file; <$fh> };
 
 	parse_plist( $text );
 	}
@@ -275,7 +260,7 @@ sub parse_plist_file {
 
 Create a plist dictionary from the hash reference.
 
-The values of the hash can only be simple scalars–not references.
+The values of the hash can only be simple scalars--not references.
 Reference values are silently ignored.
 
 Returns a string representing the hash in the plist format.
@@ -285,12 +270,9 @@ Returns a string representing the hash in the plist format.
 sub create_from_hash {
 	my $hash  = shift;
 
-	unless( ref $hash eq ref {} ) {
-		carp "create_from_hash did not get an hash reference";
-		return;
-		}
+	return unless UNIVERSAL::isa( $hash, 'HASH' );
 
-	my $string = XML_head() . Mac::PropertyList::dict->write_open . "\n";
+	my $string = "$XML_head" . Mac::PropertyList::dict->write_open . "\n";
 
 	foreach my $key ( keys %$hash ) {
 		next if ref $hash->{$key};
@@ -305,7 +287,7 @@ sub create_from_hash {
 		$string .= $bit;
 		}
 
-	$string .= Mac::PropertyList::dict->write_close . "\n" . XML_foot();
+	$string .= Mac::PropertyList::dict->write_close . "\n$XML_foot";
 
 	return $string;
 	}
@@ -314,7 +296,7 @@ sub create_from_hash {
 
 Create a plist array from the array reference.
 
-The values of the array can only be simple scalars–not references.
+The values of the array can only be simple scalars--not references.
 Reference values are silently ignored.
 
 Returns a string representing the array in the plist format.
@@ -324,12 +306,9 @@ Returns a string representing the array in the plist format.
 sub create_from_array {
 	my $array  = shift;
 
-	unless( ref $array eq ref [] ) {
-		carp "create_from_array did not get an array reference";
-		return;
-		}
+	return unless UNIVERSAL::isa( $array, 'ARRAY' );
 
-	my $string = XML_head() . Mac::PropertyList::array->write_open . "\n";
+	my $string = "$XML_head" . Mac::PropertyList::array->write_open . "\n";
 
 	foreach my $element ( @$array ) {
 		my $value = Mac::PropertyList::string->new( $element );
@@ -340,52 +319,11 @@ sub create_from_array {
 		$string .= $bit;
 		}
 
-	$string .= Mac::PropertyList::array->write_close . "\n" . XML_foot();
+	$string .= Mac::PropertyList::array->write_close . "\n$XML_foot";
 
 	return $string;
 	}
 
-=item create_from_string( STRING )
-
-Returns a string representing the string in the plist format.
-
-=cut
-
-sub create_from_string {
-	my $string  = shift;
-
-	unless( ! ref $string ) {
-		carp "create_from_string did not get a string";
-		return;
-		}
-
-	return
-		XML_head() .
-		Mac::PropertyList::string->new( $string )->write .
-		"\n" . XML_foot();
-	}
-
-=item create_from
-
-Dispatches to either C<create_from_array>, C<create_from_hash>, or
-C<create_from_string> based on the argument. If none of those fit,
-this C<croak>s.
-
-=cut
-
-sub create_from {
-	my $thingy  = shift;
-
-	return do {
-		if(      ref $thingy eq ref [] ) { &create_from_array  }
-		elsif(   ref $thingy eq ref {} ) { &create_from_hash   }
-		elsif( ! ref $thingy eq ref {} ) { &create_from_string }
-		else {
-			croak "Did not recognize argument! Must be a string, or reference to a hash or array";
-			}
-		};
-	}
-
 =item read_string
 
 =item read_data
@@ -426,18 +364,12 @@ sub read_next {
 	while( not defined $value ) {
 		croak "Couldn't read anything!" if $source->eof;
 		$_ .= $source->get_line;
-		if( s[^\s* < (string|date|real|integer|data) > \s*(.*?)\s* </\1> ][]sx ) {
+
+		if( s[^\s* < (string|date|real|integer|data) >
+			   \s*(.*?)\s* </\1> ][]sx ) {
 			$value = $Readers{$1}->( $2 );
 			}
-		elsif( s[^\s* < string / > ][]x ){
-			$value = $Readers{'string'}->( '' );
-			}
 	    elsif( s[^\s* < (dict|array) > ][]x ) {
-			# We need to put back the unprocessed text if
-			# any because the <dict> and <array> readers
-			# need to see it.
-			$source->put_line( $_ ) if defined $_ && '' ne $_;
-			$_ = '';
 			$value = $Readers{$1}->( $source );
 			}
 	    # these next two are some wierd cases i found in the iPhoto Prefs
@@ -486,13 +418,7 @@ sub read_dict {
 		}
 
 	$source->put_line( $_ );
-	if ( 1 == keys %hash && exists $hash{'CF$UID'} ) {
-	    # This is how plutil represents a UID in XML.
-	    return Mac::PropertyList::uid->integer( $hash{'CF$UID'}->value );
-	    }
-	else {
-	    return Mac::PropertyList::dict->new( \%hash );
-	    }
+	return Mac::PropertyList::dict->new( \%hash );
 	}
 
 =item read_array
@@ -527,52 +453,21 @@ sub read_data {
 	return Mac::PropertyList::data->new( $string );
 	}
 
-=back
-
-=head2 Things that write
-
-=over 4
-
-=item XML_head
-
-Returns a string that represents the start of the PList XML.
-
-=cut
-
-sub XML_head () {
-	<<"XML";
-<?xml version="1.0" encoding="UTF-8"?>
-<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
-<plist version="1.0">
-XML
-	}
-
-=item XML_foot
-
-Returns a string that represents the end of the PList XML.
-
-=cut
-
-sub XML_foot () {
-	<<"XML";
-</plist>
-XML
-	}
-
 =item plist_as_string
 
 Return the plist data structure as XML in the Mac Property List format.
 
 =cut
 
-sub plist_as_string {
+sub plist_as_string
+	{
 	my $object = CORE::shift;
 
-	my $string = XML_head();
+	my $string = $XML_head;
 
 	$string .= $object->write . "\n";
 
-	$string .= XML_foot();
+	$string .= $XML_foot;
 
 	return $string;
 	}
@@ -580,16 +475,20 @@ sub plist_as_string {
 =item plist_as_perl
 
 Return the plist data structure as an unblessed Perl data structure.
-There won't be any C<Mac::PropertyList> objects in the results. This
-is really just C<as_perl>.
+There won't be any C<Mac::PropertyList> objects in the results.
 
 =cut
 
-sub plist_as_perl { $_[0]->as_perl }
+sub plist_as_perl
+	{
+	my $object = CORE::shift;
 
-=back
+	my $string = '$VAR = ';
 
-=cut
+	$string .= $object->as_perl;
+
+	return $string;
+	}
 
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::Source;
@@ -603,34 +502,13 @@ sub eof { (not @{$_[0]->{buffer}}) and $_[0]->source_eof }
 sub get_line {
 	my $self = CORE::shift;
 
-# I'm not particularly happy with what I wrote here, but that's why
-# you shouldn't write your own buffering code! I might have left over
-# text in the buffer. This could be stuff a higher level looked at and
-# put back with put_line. If there's text there, grab that.
-#
-# But here's the tricky part. If that next part of the text looks like
-# a "blank" line, grab the next next thing and append that.
-#
-# And, if there's nothing in the buffer, ask for more text from
-# get_source_line. Follow the same rules. IF you get back something that
-# looks like a blank line, ask for another and append it.
-#
-# This means that a returned line might have come partially from the
-# buffer and partially from a fresh read.
-#
-# At some point you should have something that doesn't look like a
-# blank line and the while() will break out. Return what you do.
-#
-# Previously, I wasn't appending to $_ so newlines were disappearing
-# as each next read replaced the value in $_. Yuck.
-
 	local $_ = '';
 	while (defined $_ && /^[\r\n\s]*$/) {
 		if( @{$self->{buffer}} ) {
-			$_ .= shift @{$self->{buffer}};
+			$_ = shift @{$self->{buffer}};
 			}
 		else {
-			$_ .= $self->get_source_line;
+			$_ = $self->get_source_line;
 			}
 		}
 
@@ -676,6 +554,8 @@ sub value {
 sub type { my $r = ref $_[0] ? ref $_[0] : $_[0]; $r =~ s/.*:://; $r; }
 
 sub new {
+	#print STDERR "Got [@_]\n";
+
 	bless $_[1], $_[0]
 	}
 
@@ -769,6 +649,8 @@ package Mac::PropertyList::dict;
 use base qw(Mac::PropertyList::Container);
 
 sub new {
+	#print STDERR Data::Dumper::Dumper( $_[1] );
+
 	$_[0]->SUPER::new( $_[1] );
 	}
 
@@ -810,9 +692,7 @@ sub as_basic_data {
 	return \%dict;
 	}
 
-sub write_key   {
-	"<key>$_[1]</key>"
-	}
+sub write_key   { "<key>$_[1]</key>" }
 
 sub write {
 	my $self  = shift;
@@ -835,7 +715,8 @@ sub write {
 	return $string;
 	}
 
-sub as_perl {
+sub as_perl
+	{
 	my $self  = CORE::shift;
 
 	my %dict = map {
@@ -856,7 +737,7 @@ sub new { my $copy = $_[1]; $_[0]->SUPER::new( \$copy ) }
 
 sub as_basic_data { $_[0]->value }
 
-sub write { $_[0]->write_open . HTML::Entities::encode_entities($_[0]->value) . $_[0]->write_close }
+sub write { $_[0]->write_open . $_[0]->value . $_[0]->write_close }
 
 sub as_perl { $_[0]->value }
 
@@ -872,74 +753,6 @@ use base qw(Mac::PropertyList::Scalar);
 package Mac::PropertyList::integer;
 use base qw(Mac::PropertyList::Scalar);
 
-# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
-package Mac::PropertyList::uid;
-use base qw(Mac::PropertyList::Scalar);
-
-# The following is conservative, since the actual largest unsigned
-# integer is ~0, which is 0xFFFFFFFFFFFFFFFF on many (most?) modern
-# Perls; but it is consistent with Mac::PropertyList::ReadBinary.
-# This is really just future-proofing though, since it appears from
-# CFBinaryPList.c that a UID is carried as a hard-coded uint32_t.
-use constant LONGEST_HEX_REPRESENTABLE_AS_NATIVE => 8;	# 4 bytes
-
-# Instantiate with hex string. The string will be padded on the left
-# with zero if its length is odd. It is this string which will be
-# returned by value(). Presence of a non-hex character causes an
-# exception. We default the argument to '00'.
-sub new {
-	my ( $class, $value ) = @_;
-	$value = '00' unless defined $value;
-	Carp::croak( 'uid->new() argument must be hexadecimal' )
-		if $value =~ m/ [[:^xdigit:]] /smx;
-	substr $value, 0, 0, '0'
-		if length( $value ) % 2;
-	return $class->SUPER::new( $value );
-	}
-
-# Without argument, this is an accessor returning the value as an unsigned
-# integer, either a native Perl value or a Math::BigInt as needed.
-# With argument, this is a mutator setting the value to the hex
-# representation of the argument, which must be an unsigned integer,
-# either native Perl of Math::BigInt object. If called as static method
-# instantiates a new object.
-sub integer {
-	my ( $self, $integer ) = @_;
-	if ( @_ < 2 ) {
-		my $value = $self->value();
-		return length( $value ) > LONGEST_HEX_REPRESENTABLE_AS_NATIVE ?
-			Math::BigInt->from_hex( $value ) :
-			hex $value;
-		}
-	else {
-		Carp::croak( 'uid->integer() argument must be unsigned' )
-			if $integer < 0;
-		my $value = ref $integer ?
-			$integer->to_hex() :
-			sprintf '%x', $integer;
-		if ( ref $self ) {
-			substr $value, 0, 0, '0'
-				if length( $value ) % 2;
-			${ $self } = $value;
-			}
-		else {
-			$self = $self->new( $value );
-			}
-		return $self;
-		}
-	}
-
-# This is how plutil represents a UID in XML.
-sub write {
-	my $self = shift;
-	my $dict = Mac::PropertyList::dict->new( {
-			'CF$UID' => Mac::PropertyList::integer->new(
-				$self->integer ),
-			}
-		);
-	return $dict->write();
-	}
-
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
 package Mac::PropertyList::string;
 use base qw(Mac::PropertyList::Scalar);
@@ -964,7 +777,7 @@ sub write {
 
 	my $string = MIME::Base64::encode_base64($value);
 
-	$self->write_open . HTML::Entities::encode_entities($string) . $self->write_close;
+	$self->write_open . $string . $self->write_close;
 	}
 
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -992,11 +805,13 @@ package Mac::PropertyList::false;
 use base qw(Mac::PropertyList::Boolean);
 
 
+=back
+
 =head1 SOURCE AVAILABILITY
 
 This project is in Github:
 
-	https://github.com/briandfoy/mac-propertylist.git
+	git://github.com/briandfoy/mac-propertylist.git
 
 =head1 CREDITS
 
@@ -1022,16 +837,14 @@ all the plist junk and just play with the data.
 
 =head1 AUTHOR
 
-brian d foy, C<< <briandfoy@pobox.com> >>
-
-Tom Wyant added support for UID types.
+brian d foy, C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2004-2025, brian d foy <briandfoy@pobox.com>. All rights reserved.
+Copyright (c) 2004-2011 brian d foy.  All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
-it under the terms of the Artistic License 2.0.
+it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
