@@ -1,24 +1,22 @@
-use v5.10;
-
 package Mac::PropertyList::ReadBinary;
 use strict;
 use warnings;
 
+use vars qw( $VERSION );
+
 use Carp;
 use Data::Dumper;
 use Encode            qw(decode);
 use Mac::PropertyList;
 use Math::BigInt;
+use MIME::Base64      qw(decode_base64);
 use POSIX             qw(SEEK_END SEEK_SET);
+use XML::Entities     ();
 
-our $VERSION = '1.603_02';
-
-my $Debug = $ENV{PLIST_DEBUG} || 0;
+$VERSION = '1.35';
 
 __PACKAGE__->_run( @ARGV ) unless caller;
 
-=encoding utf8
-
 =head1 NAME
 
 Mac::PropertyList::ReadBinary - read binary property list files
@@ -32,6 +30,7 @@ Mac::PropertyList::ReadBinary - read binary property list files
 
 	my $plist = $parser->plist;
 
+
 	# use indirectly, automatically selects right reader
 	use Mac::PropertyList;
 
@@ -40,17 +39,17 @@ Mac::PropertyList::ReadBinary - read binary property list files
 =head1 DESCRIPTION
 
 This module is a low-level interface to the Mac OS X Property List
-(plist) format. You probably shouldn't use this in
-applications—build interfaces on top of this so you don't have to
+(plist) format.  You probably shouldn't use this in
+applications---build interfaces on top of this so you don't have to
 put all the heinous multi-level object stuff where people have to look
 at it.
 
 You can parse a plist file and get back a data structure. You can take
 that data structure and get back the plist as XML (but not binary
-yet). If you want to change the structure inbetween that's your
+yet).  If you want to change the structure inbetween that's your
 business. :)
 
-See L<Mac::PropertyList> for more details.
+See C<Mac::PropertyList> for more details.
 
 =head2 Methods
 
@@ -87,7 +86,8 @@ Returns the C<Mac::PropertyList> data structure.
 
 sub plist            { $_[0]->{parsed}             }
 
-sub _object_size {
+sub _object_size
+	{
 	$_[0]->_trailer->{object_count} * $_[0]->_trailer->{offset_size}
 	}
 
@@ -126,7 +126,8 @@ sub _get_filehandle {
 	$fh;
 	}
 
-sub _read_plist_trailer {
+sub _read_plist_trailer
+	{
 	my $self = shift;
 
 	seek $self->_fh, -32, SEEK_END;
@@ -141,7 +142,8 @@ sub _read_plist_trailer {
 	$self->{trailer} = \%hash;
 	}
 
-sub _get_offset_table {
+sub _get_offset_table
+	{
 	my $self = shift;
 
     seek $self->_fh, $self->_trailer->{table_offset}, SEEK_SET;
@@ -157,14 +159,18 @@ sub _get_offset_table {
 
 	$self->{offsets} = \@offsets;
 
-    if( $self->_trailer->{offset_size} == 3 ) {
+    if( $self->_trailer->{offset_size} == 3 )
+    	{
 		@offsets = map { hex } @offsets;
    	 	}
+
 	}
 
 sub _read_object_at_offset {
 	my( $self, $offset ) = @_;
 
+	my @caller = caller(1);
+
     seek $self->_fh, ${ $self->_offsets }[$offset], SEEK_SET;
 
     $self->_read_object;
@@ -173,66 +179,45 @@ sub _read_object_at_offset {
 # # # # # # # # # # # # # #
 
 BEGIN {
-
-my %singletons = (
-    0 => undef,
-    8 => Mac::PropertyList::false->new(),
-    9 => Mac::PropertyList::true->new(),
-
-    # 15 is also defined (as "fill") in the comments to Apple's
-    # implementation in CFBinaryPList.c but Apple's actual code has no
-    # support for it at all, either reading or writing, so it's
-    # probably not important to implement.
-	);
-
 my $type_readers = {
+
 	0 => sub { # the odd balls
 		my( $self, $length ) = @_;
 
-		return $singletons{ $length } if exists $singletons{ $length };
+		my %hash = (
+			 0 => [ qw(null  0) ],
+			 8 => [ qw(false 0) ],
+			 9 => [ qw(true  1) ],
+			15 => [ qw(fill 15) ],
+			);
 
-		croak ( sprintf "Unknown type byte %02X\n", $length );
+		return $hash{ $length } || [];
     	},
 
 	1 => sub { # integers
-		my( $self, $power_of_2 ) = @_;
+		my( $self, $length ) = @_;
+		croak "Integer > 8 bytes = $length" if $length > 3;
 
-		croak "Integer with <$power_of_2> bytes is not supported" if $power_of_2 > 4;
-
-		my $byte_length = 1 << $power_of_2;
+		my $byte_length = 1 << $length;
 
 		my( $buffer, $value );
 		read $self->_fh, $buffer, $byte_length;
 
-		my @formats = qw( C n N NN NNNN );
-		my @values = unpack $formats[$power_of_2], $buffer;
+		my @formats = qw( C n N NN );
+		my @values = unpack $formats[$length], $buffer;
 
-		if( $power_of_2 == 3 ) { # 64 bits
+		if( $length == 3 )
+			{
 			my( $high, $low ) = @values;
 
 			my $b = Math::BigInt->new($high)->blsft(32)->bior($low);
-			if( $b->bcmp(Math::BigInt->new(2)->bpow(63)) >= 0) {
+			if( $b->bcmp(Math::BigInt->new(2)->bpow(63)) > 0)
+				{
 				$b -= Math::BigInt->new(2)->bpow(64);
 				}
 
 			@values = ( $b );
 			}
-		elsif( $power_of_2 == 4 ) { # 128 bits
-		    # 128 bits aren't part of the public API, but apparently
-		    # they are out there.
-			my( $highest, $higher, $high, $low ) = @values;
-			my $b = Math::BigInt
-				->new($highest)
-				->blsft(32)->bior($higher)
-				->blsft(32)->bior($high)
-				->blsft(32)->bior($low);
-
-			if( $b->bcmp(Math::BigInt->new(2)->bpow(127)) >= 0) {
-				$b -= Math::BigInt->new(2)->bpow(128);
-				}
-
-			@values = ( $b );
-			}
 
 		return Mac::PropertyList::integer->new( $values[0] );
 		},
@@ -266,7 +251,7 @@ my $type_readers = {
 		$self->{MLen} += 9;
 
 		my $adjusted_time = POSIX::strftime(
-			"%Y-%m-%dT%H:%M:%SZ",
+			"%FT%H:%M:%SZ",
 			gmtime( 978307200 + $values[0])
 			);
 
@@ -288,7 +273,11 @@ my $type_readers = {
 		my( $buffer, $value );
 		read $self->_fh, $buffer, $length;
 
-		$buffer = Encode::decode( 'ascii', $buffer );
+		# pack to make it unicode
+		$buffer = XML::Entities::decode(
+			'all',
+			pack "U0C*", unpack "C*", $buffer
+			);
 
 		return Mac::PropertyList::string->new( $buffer );
 		},
@@ -299,23 +288,14 @@ my $type_readers = {
 		my( $buffer, $value );
 		read $self->_fh, $buffer, 2 * $length;
 
-		$buffer = Encode::decode( "UTF-16BE", $buffer );
+		$buffer = XML::Entities::decode(
+			'all',
+			Encode::decode( "UTF-16BE", $buffer )
+			);
 
 		return Mac::PropertyList::ustring->new( $buffer );
 		},
 
-	8 => sub { # UIDs
-		my( $self, $length ) = @_;
-
-		my $byte_length = $length + 1;
-
-		read $self->_fh, ( my $buffer ), $byte_length;
-
-		my $value = unpack 'H*', $buffer;
-
-		return Mac::PropertyList::uid->new( $value );
-		},
-
 	a => sub { # array
 		my( $self, $elements ) = @_;
 
@@ -364,14 +344,16 @@ my $type_readers = {
 		},
 	};
 
-sub _read_object {
+sub _read_object
+	{
 	my $self = shift;
-    my $buffer;
 
+    my $buffer;
     croak "read() failed while trying to get type byte! $!"
     	unless read( $self->_fh, $buffer, 1) == 1;
 
     my $length = unpack( "C*", $buffer ) & 0x0F;
+
     $buffer    = unpack "H*", $buffer;
     my $type   = substr $buffer, 0, 1;
 
@@ -398,22 +380,20 @@ Some of the ideas are cribbed from CFBinaryPList.c
 
 This project is in Github:
 
-	https://github.com/briandfoy/mac-propertylist.git
+	git://github.com/briandfoy/mac-propertylist.git
 
 =head1 CREDITS
 
 =head1 AUTHOR
 
-brian d foy, C<< <briandfoy@pobox.com> >>
-
-Tom Wyant added support for UID types.
+brian d foy, C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2004-2025, brian d foy <briandfoy@pobox.com>. All rights reserved.
+Copyright (c) 2004-2011 brian d foy.  All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
-it under the terms of the Artistic License 2.0.
+it under the same terms as Perl itself.
 
 =cut
 
